"use strict";

this.name    = "AI Trading Tips";
this.version = "2.12.2";

/* ====================================================================================
			MESSAGES DISPLAYED BY TRADING ASSISTANT

Each of the messages is an array of three items: message, summary, highlight.

message   - is the comms message to appear in the lower left of the screen.
summary   - is the one-word message that may appear at the bottom of the costs column
            and be read out by the text-to-speech function.
highlight - is the commodity type to be highlighted on the HUD.
======================================================================================= */

this.$tips = [
	["Trading Assistant remembers how long cargo has been aboard, and will remind you about nuisance cargo you’ve been unable to sell.", "", ""],
	["The Trading Assistant configuration screen shows some extra information if Autotrade is turned off.", "", ""],
	["Selecting your destination on the navigation chart before trading is a good idea.", "", ""],
	["You can set Autotrade to reserve some empty space when buying, if you anticipate scooping extra cargo en-route.", "", ""],
	["If set to buying mode, Autotrade will automatically buy the best discounts, but you must decide when to sell.", "", ""],
	["If set to selling mode, Autotrade will automatically sell cargo as soon as it is profitable, but will never buy cargo.", "", ""],
	["The docking history screen compares the price you paid for cargo to its value at this station.", "", ""],
	["The docking history screen lists stations you have visited since acquiring your current cargo.", "", ""]];

this.$moreTips = [
	["There are old traders and bold traders, but very few old, bold traders.", "", ""],
	["Wo ist der Download-Link?\nwiki.alioth.net/index.php/AI_Trading_Assistant#Download", "Achtung!", ""],
	["No one ever loses money selling liquor to the Galactic Navy.\n- Cmdr. Ranthe", "Quote", "liquor_wines"],
	["What a totally brilliant piece of equipment. This just gets better and better and better!\n- Cmdr. Styggron", "Quote", ""],
	["Buy low, sell high.", "Wisdom", ""]];

/* ====================================================================================
			TIPS REGARDING MAIN STATION PRICES

	Don't copy $expectedPrices from Autotrade script, because it might have been 
	altered to avoid or hoard particular commodities. Use a hard-coded list here.
======================================================================================= */

    this._createGalCopPriceTips = function () {
        "use strict";
        var commodities = ["food", "textiles", "radioactives", "liquor_wines", "luxuries", "computers", "machinery", "alloys", "furs", "minerals", "gold", "platinum", "gem_stones", "alien_items"];
        var prices = [50, 72, 232, 292, 902, 818, 566, 388, 705, 120, 388, 718, 196, 436]; // deci-credits
        var arrayLength = commodities.length;
        for (var i = 0; i < arrayLength; i++) {
            var message = ["", "", commodities[i], , "galcop"];
            message[0] = "The average price of " + displayNameForCommodity(commodities[i]).toLowerCase() + " at GalCop stations is " + (prices[i] / 10).toFixed(1) + " credits.";
            this.$moreTips.push(message);
        }
        this.$moreTips.push(["GalCop stations never sell " + displayNameForCommodity("alien_items").toLowerCase() + ", but are always willing to buy them, and never pay less than 21.2 credits.", "", "alien_items", , "galcop"]);
        return true;
    }

/* ====================================================================================
			TIPS REGARDING DOCKED STATION PRICES
======================================================================================= */

    this.shipDockedWithStation = function (station) {
        "use strict";
        if (!station.isMainStation) {
            this._createStationPriceTips();
        }
    }

    this.shipLaunchedFromStation = function () {
        "use strict";
        this._removeTips("this station");
    }

    this._createStationPriceTips = function () {
        "use strict";
        if (!player.ship.docked || !system.mainStation) {
            return false;
        }
        var thisMarket = player.ship.dockedStation.market;
        var thatMarket = system.mainStation.market;
        var commodities = ["food", "textiles", "radioactives", "liquor_wines", "luxuries", "computers", "machinery", "alloys", "furs", "minerals", "gold", "platinum", "gem_stones", "alien_items"];
        var arrayLength = commodities.length;
        for (var i = 0; i < arrayLength; i++) {
            var cargo = commodities[i];
            var thisPrice = thisMarket[cargo].price;
            var thatPrice = thatMarket[cargo].price;
            var message = ["", "", cargo, , "this station"];
            if (thisPrice > thatPrice) {
                message[0] = "The price of " + displayNameForCommodity(cargo).toLowerCase()
                           + " at this station is " + ((thisPrice - thatPrice) / 10).toFixed(1)
                           + " credits higher than the GalCop station in this system.";
                this.$tips.push(message);
            }
            if (thisPrice < thatPrice) {
                message[0] = "The price of " + displayNameForCommodity(cargo).toLowerCase()
                           + " at this station is " + ((thatPrice - thisPrice) / 10).toFixed(1)
                           + " credits lower than the GalCop station in this system.";
                this.$tips.push(message);
            }
        }
        return true;
    }

/* ====================================================================================
			PUBLIC FUNCTION TO ADD A TIP

	worldScripts["AI Trading Tips"]._addTip(category, message)

category  - a string used to identify your messages, e.g. "myCategory".
message   - an array containing a message in the format used by Trading Assistant (described below).
Example:

worldScripts["AI Trading Tips"]._addTip("price watch", ["This station has the lowest price for food, like ever.", "Bargain!", "food"])
======================================================================================= */

    this._addTip = function (category, message) {
        "use strict";
        if (!message) {
            log(this.name, "ERROR: _addTip function was called with no message parameter.");
            return false;
        }
        if (!category) {
            log(this.name, "ERROR: _addTip function was called with no category parameter.");
            return false;
        }
     // if (message[0] && message[1] && message[2]) { // BUG: empty "" in [1] or [2] causes this to fail.
        if (message.length > 2) {
            message[4] = category;
            this.$moreTips.push(message);
            return true;
        } else if (message[0]) {
            message[1] = "";
            message[2] = "";
            message[4] = category;
            this.$moreTips.push(message);
            return true;
        }
        log(this.name, "ERROR: _addTip function was called with a malformed message parameter.");
        return false;
    }

/* ====================================================================================
			FUNCTION TO REMOVE ALL TIPS IN GIVEN CATEGORY

	worldScripts["AI Trading Tips"]._removeTips(category)

category  - a string used to identify your messages, e.g. "myCategory".
Example:

this.shipLaunchedFromStation = function()
	{
	worldScripts["AI Trading Tips"]._removeTips("price watch")
	}
======================================================================================= */

    this._removeTips = function (category) {
        "use strict";
        if (!category) {
            log(this.name, "ERROR: _removeTips function was called with no category parameter.");
            return false;
        }
        var counter = 0, i;
        for (i = this.$tips.length - 1; i >= 0; i--) {
            if (this.$tips[i][4] && this.$tips[i][4] === category) {
                this.$tips.splice(i, 1);
                counter++;
            }
        }
        for (i = this.$moreTips.length - 1; i >= 0; i--) {
            if (this.$moreTips[i][4] && this.$moreTips[i][4] === category) {
                this.$moreTips.splice(i, 1);
                counter++;
            }
        }
        return counter;
    }

/* ====================================================================================
			RANDOMLY SELECT MESSAGE

	Randomly select one message from $tips, and move it to $moreTips.
======================================================================================= */

    this._randomTip = function () {
        "use strict";
        if (this.$tips.length == 0) {
            if (this.$moreTips.length == 0) {
                return false;
            }
            this.$tips = this.$moreTips;
            this.$moreTips = [];
        }
        var i = Math.floor(Math.random() * this.$tips.length);
        var message = this.$tips[i];
        this.$tips.splice(i, 1);
        // Always return an array with at least 3 elements,
        // and discard malformed messages...
        if (message && message.length > 2) {
            this.$moreTips.push(message);
            return message;
        } else {
            return ["", "", ""];
        }
    }

/* ====================================================================================
			REPLACE USUAL FLUMMOX RESPONSE AT START-UP

	This replaces the usual _flummoxResponse function in the Autotrade script.
======================================================================================= */

    this.startUp = function () {
        "use strict";
        if (worldScripts['AI Autotrade'] &&
            worldScripts['AI Autotrade']._flummoxResponse) {
            worldScripts['AI Autotrade']._flummoxResponse = this._flummoxResponse;
        }
        this._createGalCopPriceTips();
        delete this._flummoxResponse;
        delete this.startUp;
    }

    this._flummoxResponse = function (flummoxCount) {
        "use strict";
        if (flummoxCount > 4 && (flummoxCount & 0x01)) {
            return;
        } else if (flummoxCount > 4) {
            var message = worldScripts['AI Trading Tips']._randomTip();
            if (message) {
                worldScripts['AI Trading Assistant'].$showMessage(message[0], message[1], message[2]);
                return;
            } else {
                this._stopAutotrade();
                return;
            }
        } else if (flummoxCount > 2) {
            if (this.$tradingTimer) {
                this.$tradingTimer.stop();
            }
            worldScripts["AI Trading Assistant"]._undoDisplayMode(4);
            worldScripts['AI Trading Assistant'].$showMessage(this.$noMoreMessage[0], this.$noMoreMessage[1], this.$noMoreMessage[2]);
            return;
        } else {
            worldScripts['AI Trading Assistant'].$showMessage(this.$flummoxMessage[0], this.$flummoxMessage[1], this.$flummoxMessage[2]);
            return;
        }
    }

/* ====================================================================================
			COPYRIGHT NOTICE

	(C)2013-2014, Wildeblood. All rights reserved. 
	Free to use, but DO NOT PROMULGATE MODIFIED VERSIONS.

This was created by some fellow with the user name "Wildeblood" on the Oolite forums, 
he'll tell you his real name if you bother to ask.
To use means to play with, modify or re-use in whole or in part, and to distribute freely.
To use DOES NOT imply to use for commercial gain, which would be a breach of copyright.
To promulgate means to make official announcements.
======================================================================================= */
